#include <stdio.h>
#include <math.h>
#include <vga.h>
#include "getch.h"
#include "usleep.c"

#define MAPLENGTH 3000000	// en metres
#define MAPDEPTH -2000		// en metre
#define MAPBOX 512	// 256Ko pour la map
/*
#define MAPU (1/MAPBOX)	// dans notre unitee custom
#define TAILLEMETREH (1/MAPLENGTH)	// taille d'un metre horizontal dans notre unitee custom
#define TAILLEMETREV (1/MAPDEPTH)	// taille d'un metre vertical en unites zero
*/

#define EAUWEIGHT 1000	// poid d'un m3 d'eau
#define SUBLENGTH 77
#define SUBHEIGHT 10
#define SUBVOLUME (PI*SUBLENGTH*(SUBHEIGHT*SUBHEIGHT)/4)
#define SUBWEIGHT 5000000	// just a test
#define FIULWEIGHT 1200	// 1 m3 de fiul
#define FIULCAPACITY (SUBVOLUME/15)	// volume des reservoires en m3
#define G 9.81
#define BALASTCAPACITY (SUBVOLUME/5)
#define FROTH 100000	// coef de frottement de l'eau sur la coque
#define FROTV 1500000	// idem dans l'autre sens
#define RPMMAX 2000	// pour l'electrique
#define RPMMAXD 4000	// pour le diesel
#define KHELI 0.007	// du fait de la profondeur, ecrase le rendement de l'helice
#define FHELI 740	// coef pour le rendement de l'helice
#define FEVAC 200	// coef pour le nombre de m3 d'eau evacuable des balasts
#define KEVAC 0.01	// du fait de la profondeur, ecrase la vitesse d'evacuation de l'eau
#define FREMPLI 100	// nombre de m3 par secondes pour remplir les balasts (de flotte)
#define PROFMAX -500	// prof max au dela de laquelle il faut remonter a la nage
#define ALPHAMAX (PI/3.5)	// angle maxi pour le gouvernail horizontal
#define ALPHAVIT (ALPHAMAX/3)
#define ANGVMAX (PI/4)	// inclunaison max du sous marin (au dela = vrille)
#define ROTVIT 0.05

short int *carte;

int date=0;	// compte les secondes...

char color=1;

typedef struct {
	double x, y, z;
	} vector;	
	
typedef struct {
	vector pos, vit, acc;
	double angh, angv;
	double fiul, balast, balastvoulu;
	double rpm, rpmd;	// nombre de tour/minute
	double alpha, alphavoulu, beta, betavoulu;	// position des gouvernails horizonto et vertico
	double fluxvit;
#define DIESEL 1
#define ELECTRIK 2
#define MOTOR (DIESEL+ELECTRIK)
	char flag;	// bit 0 = 1 -> diesel on / bit 1 = 1 -> electrik on
	} sousmarin_s;

sousmarin_s sub[1] = { { { MAPLENGTH/2, MAPLENGTH/2, -30 }, { 0, 0, 0 }, { 0, 0, 0 }, 0, 0, FIULCAPACITY, 0, 0, 0, 0, 0, 0, 0, 0, 0 } }; 


void visugfx(){
	vga_setcolor(color++);
	vga_drawline((int)(sub[0].pos.x)%800,  (int)(sub[0].pos.y)%600, ((int)(sub[0].pos.x)%800)+10*sub[0].fluxvit*cos(sub[0].angh), (int)(sub[0].pos.y)%600+10*sub[0].fluxvit*sin(sub[0].angh));
}

void visugfx2(){
	vga_setcolor(color=color%16+1);
	vga_drawline((int)(sub[0].pos.x)%800,  -sub[0].pos.z, ((int)(sub[0].pos.x)%800)+10*sub[0].fluxvit*cos(sub[0].angv), -sub[0].pos.z-10*sub[0].fluxvit*sin(sub[0].angv));
}

void visu(int i){
	printf("\033[01;34mdate\033[0m : %d\n", date);
	printf("Pos : %f %f %f ! b : %.1f ! rpm : %.1f ! rmpd : %.1f\n", sub[i].pos.x, sub[i].pos.y, sub[i].pos.z, 100*sub[i].balast/BALASTCAPACITY, 100*sub[i].rpm/RPMMAX, 100*sub[i].rpmd/RPMMAXD);
	printf("Flux : %f ! Vit Vertic : %f \n", sub[i].fluxvit, sub[i].vit.z);
	printf("Orientation : %f %f ! ", sub[i].angh, sub[i].angv);
	if(sub[i].flag & 1) printf("D");
	if(sub[i].flag & 2) printf("E");
	printf("\n");
	fflush(stdout);
}

void update(int i){
	vector force, u;
	double massesubtotal, frotcoef;
	
	sub[i].pos.x += sub[i].vit.x; 
	sub[i].pos.y += sub[i].vit.y; 
	sub[i].pos.z += sub[i].vit.z;
	if(sub[i].pos.z>0) { sub[i].pos.z=sub[i].vit.z=0; sub[i].angv=0; }	// arnaque pour eviter l'effet offshore
	sub[i].vit.x += sub[i].acc.x;
	sub[i].vit.y += sub[i].acc.y;
	sub[i].vit.z += sub[i].acc.z;
	// calcul des forces
	massesubtotal=SUBWEIGHT + sub[i].fiul * FIULWEIGHT + sub[i].balast * EAUWEIGHT;
	force.z = - (massesubtotal)*G	/* poid */
		+ (SUBVOLUME*EAUWEIGHT)*G;	/* archimede */
	force.x = 0; //-FROTH*sub[i].fluxvit;
	if(sub[i].flag & ELECTRIK)
		force.x += FHELI*sub[i].rpm*exp(sub[i].pos.z*KHELI);
	if((sub[i].flag & DIESEL) && (sub[i].pos.z>-20))
		force.x += FHELI*sub[i].rpmd*exp(sub[i].pos.z*KHELI);
	if(((sub[i].flag & MOTOR)==MOTOR) && (sub[i].pos.z>-20)){
		force.x += FHELI*sub[i].rpmd*exp(sub[i].pos.z*KHELI);
		// recharger les accus
		// penser  diminuer la force de l'hlice de n%
		}
		
	u.x = cos(sub[i].angv)*cos(sub[i].angh);
	u.y = cos(sub[i].angv)*sin(sub[i].angh);
	u.z = sin(sub[i].angv);
	
	sub[i].fluxvit=sub[i].vit.x*u.x+sub[i].vit.y*u.y+sub[i].vit.z*u.z;
	
	sub[i].angh += ROTVIT*sin(sub[i].alpha)*sqrt(fabs(sub[i].fluxvit));
	if ((sub[i].angv+=ROTVIT*sin(sub[i].beta)*sqrt(fabs(sub[i].fluxvit)))>ANGVMAX || sub[i].angv<-ANGVMAX){
//		sub[i].angv = (sub[i].angv>0) ? ANGVMAX : -ANGVMAX;
//		if((sub[i].balast+=4*FREMPLI) > BALASTCAPACITY) sub[i].balast=BALASTCAPACITY;
	}
	
	frotcoef = sqrt(sub[i].vit.x*sub[i].vit.x+sub[i].vit.y*sub[i].vit.y+sub[i].vit.z*sub[i].vit.z);
	frotcoef = fabs(u.x*sub[i].vit.x + u.y*sub[i].vit.y + u.z*sub[i].vit.z)/(frotcoef?frotcoef:1) -1;
	
	sub[i].acc.x = (u.x*force.x-(FROTH+500000*frotcoef*frotcoef)*sub[i].vit.x)/massesubtotal;
	sub[i].acc.y = (u.y*force.x-(FROTH+500000*frotcoef*frotcoef)*sub[i].vit.y)/massesubtotal;
	sub[i].acc.z = (u.z*force.x-(FROTH+500000*frotcoef*frotcoef)*sub[i].vit.z)/massesubtotal;
	sub[i].acc.z += force.z/massesubtotal;
	
	if(sub[i].alphavoulu < sub[i].alpha){
		if((sub[i].alpha-=ALPHAVIT) < sub[i].alphavoulu) sub[i].alpha=sub[i].alphavoulu;
	}
	if(sub[i].alphavoulu > sub[i].alpha){
		if((sub[i].alpha+=ALPHAVIT) > sub[i].alphavoulu) sub[i].alpha=sub[i].alphavoulu;
	}
	
	if(sub[i].betavoulu < sub[i].beta){
		if((sub[i].beta-=ALPHAVIT) < sub[i].betavoulu) sub[i].beta=sub[i].betavoulu;
	}
	if(sub[i].betavoulu > sub[i].beta){
		if((sub[i].beta+=ALPHAVIT) > sub[i].betavoulu) sub[i].beta=sub[i].betavoulu;
	}
	
	printf(">>>%f<<<\n",massesubtotal);
	//
	if(sub[i].balastvoulu < sub[i].balast)
		if((sub[i].balast-=FEVAC*exp(sub[i].pos.z*KEVAC)) < sub[i].balastvoulu) sub[i].balast=sub[i].balastvoulu;
		
	if(sub[i].balastvoulu > sub[i].balast)
		if((sub[i].balast+=FREMPLI) > sub[i].balastvoulu) sub[i].balast=sub[i].balastvoulu;
		
	if(sub[i].pos.z < PROFMAX){
		printf("crack ! glou glou...\n");
		exit(1);
		}
}

void main(void){
	char c = 'r';
	int val,x,y;
	FILE *fic;
	
	vga_init();
	vga_setmode(G800x600x256);
	
	carte=malloc(sizeof(short int)*MAPBOX*MAPBOX);
	for(x=0; x<MAPBOX; x++)
		for(y=0; y<MAPBOX; y++)
			carte[x+MAPBOX*y]=-1000;


	if ((fic = fopen("data.tsv", "w")) == NULL) {
 		printf("Erreur creation data.txt\n");
 		free(carte);
		exit(1);
	}

	while(1){
		visu(0);
		visugfx();
		visugfx2();
		printf("action : ");
		c = mygetch();
		switch(c){
			case 'b':
				vga_flip();
				printf("\n %% de balast : ");
				scanf("%d",&val);
				printf("\n");
				sub[0].balastvoulu = val * BALASTCAPACITY / 100;
				vga_flip();
				break;
			case 'q': printf("\n"); fclose(fic); exit(1);
			case 'm':
				vga_flip();
				printf("\n %% de regime moteur : ");
				scanf("%d",&val);
				printf("\n");
				sub[0].rpm = val*RPMMAX/100;
				vga_flip();
				break;
			case 'n':
				vga_flip();
				printf("\n %% de regime moteur : ");
				scanf("%d",&val);
				printf("\n");
				sub[0].rpmd = val*RPMMAXD/100;
				vga_flip();
				break;
			case 'd':
				sub[0].flag ^= 1;
				break;
			case 'e':
				sub[0].flag ^= 2;
				break;
			case 'h':
				vga_flip();
				printf("\n %% horizontal : ");
				scanf("%d",&val);
				printf("\n");
				sub[0].alphavoulu = val * ALPHAMAX / 100;
				vga_flip();
				break;
			case 'v':
				vga_flip();
				printf("\n %% vertical : ");
				scanf("%d",&val);
				printf("\n");
				sub[0].betavoulu = val * ALPHAMAX / 100;
				vga_flip();
				break;
				
			default:
				fprintf(fic, "%d %f %f %f %f %f\n", date, sub[0].pos.x, sub[0].pos.y, sub[0].pos.z/50, sub[0].fluxvit, sub[0].balast);
				break;
			}
		update(0);
		date++;
	}
	
	free(carte);
	printf("\n");
}
